/********************************************************/
/*                                                      */
/*               OO7 Benchmark                          */
/*                                                      */
/*              COPYRIGHT (C) 1993                      */
/*                                                      */
/*                Michael J. Carey 		        */
/*                David J. DeWitt 		        */
/*                Jeffrey Naughton 		        */
/*               Madison, WI U.S.A.                     */
/*                                                      */
/*	         ALL RIGHTS RESERVED                    */
/*                                                      */
/********************************************************/


////////////////////////////////////////////////////////////////////////////
//
// Assembly Methods
//
////////////////////////////////////////////////////////////////////////////

#include <libc.h>
#include <stdio.h>

#include "oo7.h"
#include "globals.h"
#include "GenParams.h"
#include "VarParams.h"
#include "baidlist.h"

extern o_4b	nextComplexAssemblyId;
extern o_4b	nextBaseAssemblyId;
extern o_4b	debugMode;
extern char*	types[NumTypes];

extern BAIdList* Private_cp;
extern BAIdList* Shared_cp;

Implement PClassObj<ComplexAssembly>;
Implement PClassObj<Assembly>;
Implement PClassObj<BaseAssembly>;

////////////////////////////////////////////////////////////////////////////
//
// Assembly Constructor
//
////////////////////////////////////////////////////////////////////////////

Assembly::~Assembly()
{
}

////////////////////////////////////////////////////////////////////////////
//
// ComplexAssembly Constructor
//
////////////////////////////////////////////////////////////////////////////

ComplexAssembly::ComplexAssembly(o_4b asId, Link<Module> module,
    Link<ComplexAssembly> parentAssembly, o_4b levelNo)
{
    o_word 	i;
    o_4b	nextId;

    if (debugMode) {
        printf("ComplexAssembly::ComplexAssembly(asId = %d, levelNo = %d)\n",
                asId, levelNo);
    }

    // initialize the simple stuff
    o_4b	typeNo = (o_4b) (random() % NumTypes);

    strncpy(type, types[typeNo], TypeSize);
    buildDate = MinAssmDate + (o_4b)(random() % (MaxAssmDate-MinAssmDate+1));
    id = asId;
    assmType = Complex;
    superAssembly = parentAssembly;

    // recursively create subassemblies for this complex assembly
    for (i = 0; i < NumAssmPerAssm; i++) 
    {
        if (levelNo < NumAssmLevels-1) 
	{
	    // create a complex assembly as the subassembly
            nextId = nextComplexAssemblyId++;
//	    subAssemblies[i] = new Persistent
//			ComplexAssembly(nextId, module, this, levelNo+1);
	    subAssemblies.add( new Persistent
			ComplexAssembly(nextId, module, this, levelNo+1));
        } 
	else 
	{
	    // create a base assembly as the subassembly
            nextId = nextBaseAssemblyId++;

//	    subAssemblies[i] = new Persistent
//			BaseAssembly(nextId, module, this);
	    subAssemblies.add( new Persistent
			BaseAssembly(nextId, module, this));
        }
    }
}


ComplexAssembly::~ComplexAssembly()
{
}


////////////////////////////////////////////////////////////////////////////
//
// BaseAssembly Constructor
//
////////////////////////////////////////////////////////////////////////////

BaseAssembly::BaseAssembly(o_4b asId, Link<Module> module,
    Link<ComplexAssembly> parentAssembly) :

//    ComponentsPriv(NumCompPerAssm), ComponentsShar(NumCompPerAssm)
    ComponentsPriv(), ComponentsShar()
{

    o_4b		compId;
    Link<BaseAssembly>	self(this);

    if (debugMode) {
        printf("BaseAssembly::BaseAssembly(asId = %d, levelNo = %d)\n",
                asId, NumAssmLevels);
    }

    // initialize the simple stuff
    o_4b		typeNo = (o_4b) (random() % NumTypes);

    strncpy(type, types[typeNo], TypeSize);
    buildDate = MinAssmDate + (o_4b)(random() % (MaxAssmDate-MinAssmDate+1));
    id = asId;
    assmType = Base;
    superAssembly = parentAssembly;

    // all Base Assemblies are added to the module
    module->assemblies.add(self);

    module->dirty();

    // get access to the design library containing composite parts
    o_word	i;
    o_4b	lowCompId = (module->id - 1) * NumCompPerModule + 1;
    o_4b	compIdLimit = NumCompPerModule;

    // first select the private composite parts for this assembly
    if (debugMode) { printf("[Private parts = { "); }
    for (i = 0; i < NumCompPerAssm; i++) 
    {
	compId = lowCompId + (o_4b)(random() % compIdLimit);
	// keep track of which CP uses this base assembly as private
	Private_cp[compId].insert(self); 
        if (debugMode) { printf("%d ", compId); }
    }

    // next select the shared composite parts for this assembly
    if (debugMode) { printf("}, Shared parts = { "); }
    for (i = 0; i < NumCompPerAssm; i++) 
    {
	compId = (o_4b)(random() % TotalCompParts) + 1;
	// uses this base assembly as shared
	Shared_cp[compId].insert(self);	// keep track of which CP
        if (debugMode) { printf("%d ", compId); }
    }
    if (debugMode) { printf("}\n"); }
}


BaseAssembly::~BaseAssembly()
{
    ComponentsPriv.release();
    ComponentsShar.release();
}


//////////////////////////////////////////////////////////////////////////
// 
// DoNothing method for Assemblies 
// 
//////////////////////////////////////////////////////////////////////////

void Assembly::DoNothing ()
{
    if (id < 0) 
        printf("DoNothing - negative id\n");

    if (debugMode) {
        printf("==> DoNothing(id = %d, type = %s, buildDate = %d)\n",
                               id, type, buildDate);
    }
}
