/********************************************************/
/*                                                      */
/*               OO7 Benchmark                          */
/*                                                      */
/*              COPYRIGHT (C) 1993                      */
/*                                                      */
/*                Michael J. Carey 		        */
/*                David J. DeWitt 		        */
/*                Jeffrey Naughton 		        */
/*               Madison, WI U.S.A.                     */
/*                                                      */
/*	         ALL RIGHTS RESERVED                    */
/*                                                      */
/********************************************************/

#include <libc.h>
#include <stdio.h>

#include "oo7.h"
#include "globals.h"
#include "GenParams.h"
#include "BenchParams.h"
#include "VarParams.h"
#include <cxxcls/containe.h>
#include <cxxcls/vdiction.h>

//////////////////////////////////////////////////////////////////
//
// Insert - Create NumNewCompParts new composite parts (which
// means also creating a set of AtomicParts for each.)  Add
// a reference to each new composite part from a randomly chosen
// base assembly (a different random choice of base assembly for
// each new composite part.)
//
//////////////////////////////////////////////////////////////////

extern o_4b	debugMode;
extern o_4b	nextAtomicId;
extern o_4b	nextCompositeId;
extern o_4b	nextBaseAssemblyId;

//////////////////////////////////////////////////////////////////
//
// Routine to do Insert
//
//////////////////////////////////////////////////////////////////

void insert1()
{
    // now create the desired number of new composite parts, adding each
    // one as a (private) composite part that a randomly selected base
    // assembly now wishes to use

    o_4b 		compId;
    o_4b 		assmId;
    Link<CompositePart>	compH;
    Link<BaseAssembly>	assmH;

    if (debugMode) {
	printf("In Insert, nextCompositeId = %d:\n", nextCompositeId);
	printf("In Insert, nextAtomicId = %d:\n", nextAtomicId);
	printf("In Insert, nextBaseAssemblyId = %d:\n", nextBaseAssemblyId);
    }

    for (o_4b i = 0; i < NumNewCompParts; i++) {

	// add a new composite part to the design library
        if (debugMode) {
	    printf("In Insert, making composite part %d:\n", nextCompositeId);
	}

        compId = nextCompositeId++;
	compH = new Persistent CompositePart(compId);

	// randomly select a base assembly that should use it and figure
	// out which module it resides in
        o_4b	baseAssembliesPerModule = (nextBaseAssemblyId-1)/TotalModules;

	assmId = (o_4b)(random() % (nextBaseAssemblyId-1)) + 1;

	// now locate the appropriate base assembly object within its module
	LinkVstr<BaseAssembly>	bases;

	bases = PClassObject<BaseAssembly>::Object().select(
		    NULL, FALSE, PAttribute("Assembly::id") == (o_4b)assmId);
	if ( bases.is_empty() ) 
	{
            fprintf(stderr, "ERROR: Can't find base assembly %d\n", assmId);
            exit(1);
        }
	assmH = bases[0];
	bases.release();

        // finally, add the newly created composite part as a privately used
        // member of the randomly selected base assembly

	// first add this assembly to the list of assemblies in which
	// this composite part is used as a private member
	(compH->usedInPriv).add(assmH);
// JGM 93/11/8 don't need to mark dirty, since compH is a new object

	// then add the composite part compH to the list of private parts used
	// in this assembly
	(assmH->ComponentsPriv).add(compH);
// JGM 93/11/4 Added next line
	assmH->dirty();

        if (debugMode) {
	    printf("[just made it be used by base assembly %d]\n", assmId);
        }
    }
}
