/********************************************************/
/*                                                      */
/*               OO7 Benchmark                          */
/*                                                      */
/*              COPYRIGHT (C) 1993                      */
/*                                                      */
/*                Michael J. Carey 		        */
/*                David J. DeWitt 		        */
/*                Jeffrey Naughton 		        */
/*               Madison, WI U.S.A.                     */
/*                                                      */
/*	         ALL RIGHTS RESERVED                    */
/*                                                      */
/********************************************************/

////////////////////////////////////////////////////////////////////////////
//
// Module Methods
//
////////////////////////////////////////////////////////////////////////////

#include <libc.h>
#include <stdio.h>
#include <strings.h>

#include "oo7.h"
#include "globals.h"
#include "GenParams.h"
#include "VarParams.h"

extern o_4b	nextComplexAssemblyId;
extern o_4b	debugMode;
extern char*	types[NumTypes];

Implement PClassObj<Module>;

////////////////////////////////////////////////////////////////////////////
//
// Module Constructor
//
////////////////////////////////////////////////////////////////////////////

Module::Module(o_4b modId) : id(modId)
{
    Link<Module>	self(this);

    if (debugMode) {
        printf("Module::Module(modId = %d)\n", modId);
    }

    // initialize the simple stuff
    o_4b	typeNo = (o_4b) (random() % NumTypes);

    strncpy(type, types[typeNo], TypeSize);
    buildDate = MinModuleDate +
	(o_4b)(random() % (MaxModuleDate-MinModuleDate+1));

    // Build a manual
    manual = new Persistent Manual(id, self);

    // Reserve space for assemblies
    o_4b	size = NumAssmPerAssm;

    for (o_word n = 1; n < NumAssmLevels; n++)
	size *= NumAssmPerAssm;

//    assemblies.capacity(size);

    // now create the assemblies for the module
    if (NumAssmLevels > 1) {
        o_4b	assmId = nextComplexAssemblyId++;

        designRoot = new Persistent ComplexAssembly(assmId, self, NULL, 1);
    }
}

////////////////////////////////////////////////////////////////////////////
//
// Module Destructor
//
////////////////////////////////////////////////////////////////////////////

Module::~Module()
{
   if (debugMode) {
	printf("Module::~Module(id = %d)\n", id);
   }
}


////////////////////////////////////////////////////////////////////////////
//
// Module::scanManual() returns number of occurrences of the 
// character 'I' in the module's manual.
//
////////////////////////////////////////////////////////////////////////////

o_4b	Module::scanManual() 
{
  if ( manual.is_null() ) {
      fprintf(stderr, "scanManual: manual is null.\n");
      return -1;
  } else {
    return (manual->searchText('I'));
  }
}

////////////////////////////////////////////////////////////////////////////
//
// Module::firstLast() returns 1 if the first and last characters
// in the manual are the same, zero otherwise.
//
////////////////////////////////////////////////////////////////////////////

o_4b	Module::firstLast() 
{
  return (manual->firstLast());
}
