
#include <libc.h>
#include <stdio.h>
#include <stdlib.h>
#include <sys/time.h>
#include <sys/resource.h>

#include "oo7.h"
#include "globals.h"
#include "GenParams.h"
#include "VarParams.h"
#include "BenchParams.h"
#include "stats.h"
#include "stats_server.h"
#include "baidlist.h"


extern "C" char* getenv(const char*);

extern o_4b     RealWork;       // set to one to make DoNothings do work.
extern o_4b     WorkAmount;     // controls how much work DoNothings do

extern void     SetParams(char* configFileName);



extern void     Purge();

extern double   ComputeWallClockTime(struct timeval *startWallTime,
                                     struct timeval *endWallTime);

extern double   ComputeUserTime(struct rusage *startUsage,
                                struct rusage *endUsage);

extern double   ComputeSystemTime(struct rusage *startUsage,
                                  struct rusage *endUsage);

// to keep make happy
// and used in Insert!
BAIdList* Private_cp;
BAIdList* Shared_cp;

o_4b    nextAtomicId=0;
o_4b    nextCompositeId=0;
o_4b    nextComplexAssemblyId=0;
o_4b    nextBaseAssemblyId=0;
o_4b    nextModuleId = TotalModules;
o_4b    debugMode = FALSE;

struct rusage startUsage, endUsage;
struct timeval startWallTime;
struct timeval endWallTime;
struct timezone ignoreTimeZone;
struct timeval startWarmTime;

char    *types[NumTypes] = {
        "type000", "type001", "type002", "type003", "type004",
        "type005", "type006", "type007", "type008", "type009"
    };


void NewParseCommandLine(o_4b argc,
                      char* argv[],
                      o_4b& numXacts,
                      o_4b& numOps,
                      o_4b& RP,
                      o_4b& RS,
                      o_4b& WP,
                      o_4b& WS,
                      o_4b& clientNum)
{
        printf("Call was: ");
        for (int foo = 0; foo < argc; foo++) {
                printf("%s  ", argv[foo]);
        }
        printf("\n");

        if (strcmp(argv[argc-1], "-d") == 0) debugMode = TRUE;
        else debugMode = FALSE;
        if (argc < 8)
        {
            fprintf(stderr, "Usage: %s <configFileName> <xcount> <ocount> <rp> <rs
> <wp> <ws> <client> [-d]\n", argv[0]);
            fprintf(stderr, "xcount = number of transactions to run.\n");
            fprintf(stderr, "ocount = number of ops per transaction.\n");
            fprintf(stderr, "client = unique integer client number.\n");
            exit(1);
        }

        if ( ((numXacts  = atoi(argv[2])) <= 0) ||
             ((numOps    = atoi(argv[3])) <= 0) ||
             ((RP        = atoi(argv[4])) <  0) ||
             ((RS        = atoi(argv[5])) <  0) ||
             ((WP        = atoi(argv[6])) <  0) ||
             ((WS        = atoi(argv[7])) <  0) ||
             ((clientNum = atoi(argv[8])) <= 0) ||
             (clientNum > TotalModules) )
        {
            fprintf(stderr, "Usage: %s <configFileName> <xcount> <ocount> <op> <cl
ient> [-d]\n", argv[0]);
            fprintf(stderr, "xcount = %d, ocount = %d, clientNum = %d\n", numXacts
, numOps, clientNum);
            fprintf(stderr, "xcount and ocount must be greater than 0,\n");
            fprintf(stderr, "and client must not exceed the number of modules!\n")
;
            exit(1);
        }

}

#define DBNAME  "oo7db"
#define SNAME   "BenchDB"

main(o_4b argc, char* argv[])
{
    Module*             moduleH;

    ProcessStats        totalTime;
    ServerStats         totalSrvTime;

    char resultText[200]; 

    BenchmarkOp whichOp = PrivateRead;

        // initialize parameters for benchmark.
        if (SetParams(argv[1]) == -1)
        {
            fprintf(stderr, "Usage: %s <configFileName> <xcount> <ocount> <op> <cl
ient> [-d]\n", argv[0]);
            exit(1);
        }

     o_4b numXacts = 1;
     o_4b numOps = 1;
     o_4b RP,RS,WP,WS;
     o_4b clientNo = 0;
     o_4b numRestarts = 0;

     

     NewParseCommandLine(argc, argv, numXacts, numOps, RP,RS,WP,WS, clientNo);

     o_4b moduleId =  clientNo;
     LinkVstr<Module>    modules;

     
        // initialize the random number generator in a way that will
        // avoid lockstep access behavior if N of these run concurrently
        pid_t pid;
        char state[8];
        pid = getpid();
        initstate((int) pid, state, 8);

        // initialize timing info
        MultiStats timingData(numXacts);
        totalTime.Start();
        totalSrvTime.Start();
        // finally, repeatedly run the darn thing
          ::dom->beginsession(DBNAME,SNAME);

        modules.release();
        modules = PClassObject<Module>::Object().select(
                    NULL, FALSE, PAttribute("Module::id") == moduleId);
        moduleH = modules[0];
        if ( moduleH == NULL )
        {
            fprintf(stderr, "ERROR: Unable to access Module #%d.\n", moduleId);
            exit(1);
        }

        for (o_4b xactNo = 0; xactNo < numXacts; xactNo++)
        {
            o_4b rp=0;
            o_4b rs=0;
            o_4b wp=0;
            o_4b ws=0;

            // get starting wall clock time and usage values
            gettimeofday(&startWallTime, &ignoreTimeZone);
            getrusage(RUSAGE_SELF, &startUsage);

            // start a new transaction
            
       xbegin : 
         
            try
            {
               
              // seed the random number generator in a repeatable way (in
              // case the transaction aborts and has to be executed again)
              initstate((int) pid + xactNo, state, 8);

            for (o_4b opNo = 0; opNo < numOps; opNo++)
            {
                o_4b count = 0;
                o_4b docCount = 0;
                o_4b charCount = 0;
                o_4b replaceCount = 0;

                o_4b selectOp;

                selectOp=random() % 100;
                // printf("op : %d\n",selectOp);

                if (selectOp < RP)
                  whichOp=PrivateRead;
                else
                if (selectOp < (RP+RS))
                  whichOp=SharedRead;
                else
                if (selectOp < (RP+RS+WP))
                   whichOp=PrivateWrite;
                else
                   whichOp=SharedWrite;
        

                switch (whichOp) {
                    case PrivateRead:
                        count = moduleH->newtraverse(whichOp);
                        rp++;
                        sprintf(resultText, "xactNo = %d, opNo = %d: PrivateRead touched %d atomic parts.\n", xactNo, opNo, count);
                        break;
                    case SharedRead:
                        count = moduleH->newtraverse(whichOp);
                        rs++;
                        sprintf(resultText, "xactNo = %d, opNo = %d: Shared touched %d atomic parts.\n", xactNo, opNo, count);
                        break;
                    case PrivateWrite:
                        count = moduleH->newtraverse(whichOp);
                        wp++;
                        sprintf(resultText, "xactNo = %d, opNo = %d: PrivateWrite touched %d atomic parts.\n", xactNo, opNo, count);
                        break;
                    case SharedWrite:
                        count = moduleH->newtraverse(whichOp);
                        ws++;
                        sprintf(resultText, "xactNo = %d, opNo = %d: SharedWrite touched %d atomic parts.\n", xactNo, opNo, count);
                        break;
                    default:
                        fprintf(stderr, "Operation not available yet.\n");
                        exit(1);
                }
            }

            printf("rp=%d, rs=%d, wp=%d, ws=%d\n",rp,rs,wp,ws);
            ::dom->commit();

            gettimeofday(&endWallTime, &ignoreTimeZone);
            getrusage(RUSAGE_SELF, &endUsage);
            timingData.AddToStats(
               ComputeWallClockTime(&startWallTime, &endWallTime),
               ComputeUserTime(&startUsage, &endUsage),
               ComputeSystemTime(&startUsage, &endUsage));

          } /* end of try block */

          catch (PError& err)
          {
             if (err.errnum == SM_LOCK_DEADLOCK)
                  {
                      numRestarts++;
                      goto xbegin;
                  }
          }
          end_try;
        } 

        totalTime.Stop();
        totalSrvTime.Stop();
        printf("SUMMARY OF MULTI-USER TIMING RESULTS:\n");
        timingData.PrintStats();
        printf("(Number of transaction restarts: %d)\n", numRestarts);
        fprintf(stdout, "Total stats (client,server):\n");
        totalTime.PrintStatsHeader(stdout);
        totalTime.PrintStats(stdout, "TotalCli");
        totalSrvTime.PrintStats(stdout, "TotalSrv");

        // Exit
        exit(0);
}

