/********************************************************/
/*                                                      */
/*               OO7 Benchmark                          */
/*                                                      */
/*              COPYRIGHT (C) 1993                      */
/*                                                      */
/*                Michael J. Carey 		        */
/*                David J. DeWitt 		        */
/*                Jeffrey Naughton 		        */
/*               Madison, WI U.S.A.                     */
/*                                                      */
/*	         ALL RIGHTS RESERVED                    */
/*                                                      */
/********************************************************/

#include <libc.h>
#include <string.h>
#include <stdio.h>

#include "oo7.h"
#include "BenchParams.h"
#include "GenParams.h"
#include "globals.h"

//////////////////////////////////////////////////////////////////
//
// Traverse  - DFS traverse module.  Upon reaching a base assembly,
// visit all referenced composite parts.  At each composite part,
// take an action that depends on which traversal variant has been
// requested.
//
///////////////////////////////////////////////////////////////////

extern o_4b	debugMode;
extern void	PrintOp(BenchmarkOp op);

//////////////////////////////////////////////////////////////////
//
// Module Method for Traversal 
//
//////////////////////////////////////////////////////////////////

o_4b Module::newtraverse(BenchmarkOp op)
{
    if (debugMode) {
        printf("Module::newtraverse(id = %d, op = ", id);
	PrintOp(op);
	printf(")\n");
    }

    // group read the base assemblies
    ::dom->greadobjs(assemblies, NULL);

    // now traverse the assembly hierarchy
    return designRoot->newtraverse(op);
}


//////////////////////////////////////////////////////////////////
//
// ComplexAssembly Method for Traversal
//
//////////////////////////////////////////////////////////////////

o_4b ComplexAssembly::traverse(BenchmarkOp op)
{
    if (debugMode) {
        printf("\tComplexAssembly::traverse(id = %d, op = ", id);
	PrintOp(op);
	printf(")\n");
    }

    // traverse each of the assembly's subassemblies
  //  o_4b	size = subAssemblies.size();
    o_4b	count = 0;

   // for (o_word i = 0; i < size; i++ ) {
   //	count += subAssemblies[i]->traverse(op);
   // }
 // randomly walk down the assembly hierarchy

    o_4b randPath = (int) (random() % NumAssmPerAssm);
     
    count+=subAssemblies[randPath]->newtraverse(op); 
     
    return count;
}


//////////////////////////////////////////////////////////////////
//
// BaseAssembly Method for Traversal 
//
//////////////////////////////////////////////////////////////////


o_4b BaseAssembly::newtraverse(BenchmarkOp op)
{
    o_4b	size = ComponentsPriv.size();
    o_4b	count = 0;
    o_4b        randPath;

    if (debugMode) {
        printf("\t\tBaseAssembly::newtraverse(id = %d, op =  ", id);
	PrintOp(op);
	printf(" Privs = %d ", size);
	printf(")\n");
    }

    // establish iterator of private composite parts
//    for (o_word i = 0; i < size; i++) {
//	count += ComponentsPriv[i]->traverse(op);
//    }

     randPath = (int) (random() % NumCompPerAssm); 

     if (op == PrivateRead)
       count += ComponentsPriv[randPath]->traverse(Trav1);
     else
     if (op == PrivateWrite)
       count += ComponentsPriv[randPath]->traverse(Trav2b);
     else
     if (op == SharedRead)
       count += ComponentsShar[randPath]->traverse(Trav1);
     else
     if (op == SharedWrite)
       count += ComponentsShar[randPath]->traverse(Trav2b);
    return count;
}

