/********************************************************/
/*                                                      */
/*               OO7 Benchmark                          */
/*                                                      */
/*              COPYRIGHT (C) 1993                      */
/*                                                      */
/*                Michael J. Carey 		        */
/*                David J. DeWitt 		        */
/*                Jeffrey Naughton 		        */
/*               Madison, WI U.S.A.                     */
/*                                                      */
/*	         ALL RIGHTS RESERVED                    */
/*                                                      */
/********************************************************/

#include <libc.h>
#include <stdio.h>

#include "oo7.h"
#include "globals.h"
#include "GenParams.h"
#include "BenchParams.h"
#include "VarParams.h"

//////////////////////////////////////////////////////////////////
//
// Query #1 - randomly choose "Query1RepeatCnt" atomic parts by 
// lookup on their id field.  An index can be used for the
// actual lookup.  
//
// This is the "hand coded" version --- no SQL.
//
/////////////////////////////////////////////////////////////////

extern o_4b	debugMode;

//////////////////////////////////////////////////////////////////
//
// Routine to do Query #1
//
//////////////////////////////////////////////////////////////////

o_4b query1()
{
    o_4b	partId;

    // set random seed so "hot" runs are truly hot
    srandom(1);

    // now randomly select parts via partId index and process them
    for (o_word i = 0; i < Query1RepeatCnt; i++) {
	AtomicPart*	atomicPart;

	// generate part id and lookup part
	partId = (o_4b)(random() % TotalAtomicParts) + 1;

	LinkVstr<AtomicPart> atoms = PClassObject<AtomicPart>::Object().select(
		    NULL, FALSE, PAttribute("AtomicPart::id") == partId);
	atomicPart = atoms[0];
	atoms.release();
	if ( atomicPart == NULL ) {
            fprintf(stderr, "ERROR: Unable to find atomic part %d\n", partId);
            exit(1);
        }
        if (debugMode) {
            printf("    In Query1, partId = %d:\n", partId);
        }
	// process part by calling the null procedure
        atomicPart->DoNothing();
    }
    return Query1RepeatCnt;
}
