/********************************************************/
/*                                                      */
/*               OO7 Benchmark                          */
/*                                                      */
/*              COPYRIGHT (C) 1993                      */
/*                                                      */
/*                Michael J. Carey 		        */
/*                David J. DeWitt 		        */
/*                Jeffrey Naughton 		        */
/*               Madison, WI U.S.A.                     */
/*                                                      */
/*	         ALL RIGHTS RESERVED                    */
/*                                                      */
/********************************************************/

#include <libc.h>
#include <stdio.h>

#include "oo7.h"
#include "globals.h"
#include "GenParams.h"
#include "BenchParams.h"
#include "Iterator.h"

//////////////////////////////////////////////////////////////////
//
// Query #3 - 10% selection on AtomicParts via build date (the
// most recent 10% of AtomicParts.)  A B+ tree index is used.
//
// This is the hand-coded (no SQL) version.
//
/////////////////////////////////////////////////////////////////

extern o_4b	debugMode;

static Iterator	partI;

//////////////////////////////////////////////////////////////////
//
// Routine to do Query #3
//
//////////////////////////////////////////////////////////////////

o_4b query3()
{
    // choose a range of dates with the appropriate selectivity
    o_4b dateRange = (Query3Percent * (MaxAtomicDate - MinAtomicDate)) / 100;
    o_4b lowerDate = MaxAtomicDate - dateRange;

    if (debugMode) {
        printf("    Running Query3 with dateRange = %d, lowerDate = %d\n",
		    dateRange, lowerDate);
    }

    AtomicPart*	part;
    o_4b	count = 0;
    PPredicate	pred =  PAttribute("AtomicPart::buildDate") <= MaxAtomicDate &&
			PAttribute("AtomicPart::buildDate") >= lowerDate;

    partI.select(Atomic, pred);
    for (part = (Link<AtomicPart>)partI.first(); part != NULL;
	 part = (Link<AtomicPart>)partI.next()) {
	if (debugMode) {
	    printf("    In Query3, partId = %d, buildDate = %d\n",
			part->id, part->buildDate);
	}

	// process qualifying parts by calling the null procedure
	if (part->buildDate >= lowerDate) {
	    part->DoNothing();
	    count++;
	}
	else // shouldn't happen
	{  
	    printf("error:    In Query3, partId = %d, buildDate = %d\n",
			part->id, part->buildDate);
	}
    }
    return count;
}
