/********************************************************/
/*                                                      */
/*               OO7 Benchmark                          */
/*                                                      */
/*              COPYRIGHT (C) 1993                      */
/*                                                      */
/*                Michael J. Carey 		        */
/*                David J. DeWitt 		        */
/*                Jeffrey Naughton 		        */
/*               Madison, WI U.S.A.                     */
/*                                                      */
/*	         ALL RIGHTS RESERVED                    */
/*                                                      */
/********************************************************/

#include <libc.h>
#include <string.h>
#include <stdio.h>

#include "oo7.h"
#include "GenParams.h"
#include "VarParams.h"
#include "globals.h"

/////////////////////////////////////////////////////////////////
//
// Traversal #7.  Pick a random atomic part; traverse from the 
// part to its containing composite part P.  From P, traverse
// backwards up to all base assemblies referencing P; for each
// such base assembly, traverse upward through the assembly
// hierarchy until the root is reached.  This traversal coupled
// with traversal #1 ensures
// that all pointers are bi-directional.
//
/////////////////////////////////////////////////////////////////

extern o_4b	debugMode;

//////////////////////////////////////////////////////////////////
//
// Routine to initiate Traversal #7
//
//////////////////////////////////////////////////////////////////

o_4b traverse7()
{
    // get access to the design library containing atomic parts

    // prepare to select a random atomic part via the id index
    // now randomly select a part and traverse upwards from there

    // set random seed so "hot" runs are truly hot
    srandom(1);

    o_4b			partId = (o_4b)(random()%TotalAtomicParts) + 1;
    LinkVstr<AtomicPart>	atomicParts =
	PClassObject<AtomicPart>::Object().select(
		NULL, FALSE, PAttribute("AtomicPart::id") == partId);
    AtomicPart*			atomicPart = atomicParts[0];

    atomicParts.release();
    if ( atomicPart == NULL ) {
        fprintf(stderr, "ERROR: Unable to find atomic part %d\n", partId);
        exit(1);
    }

    if (debugMode) {
        printf("  In Traverse7, starting at atomic part %d\n", partId);
    }
    return atomicPart->partOf->traverse7();
}

//////////////////////////////////////////////////////////////////
//
// CompositePart Method for Traversal #7
//
//////////////////////////////////////////////////////////////////

o_4b CompositePart::traverse7()
{
    if (debugMode) {
        printf("\tCompositePart::traverse7(id = %d)\n", id);
    }

    // search up the design hierarchy (along the private path)
    o_4b	count = 0;
    o_4b	size = usedInPriv.size();
    PartIdSet	visitedBaseIds;
    PartIdSet	visitedComplexIds;

    // establish iterator of private base assemblies
    for (o_word i = 0; i < size; i++) {
	BaseAssembly*	baseAssembly = usedInPriv[i]; 

        if (!visitedBaseIds.contains(baseAssembly->id)) {
            visitedBaseIds.insert(baseAssembly->id);
	    count += baseAssembly->traverse7(visitedComplexIds);
	}
    }
    return count;
}

//////////////////////////////////////////////////////////////////
//
// BaseAssembly Method for Traversal #7
//
//////////////////////////////////////////////////////////////////

o_4b BaseAssembly::traverse7(PartIdSet& visitedComplexIds)
{
    if (debugMode) {
        printf("\t\tBaseAssembly::traverse7(id = %d)\n", id);
    }

    // visit this assembly and move on up the design hierarchy
    o_4b	count = 1;

    DoNothing();
    if (!visitedComplexIds.contains(superAssembly->id)) {
        count += superAssembly->traverse7(visitedComplexIds);
    }
    return count;
}

//////////////////////////////////////////////////////////////////
//
// ComplexAssembly Method for Traversal #7
//
//////////////////////////////////////////////////////////////////

o_4b ComplexAssembly::traverse7(PartIdSet& visitedComplexIds)
{
    if (debugMode) {
        printf("\t\t\tComplexAssembly::traverse7(id = %d)\n", id);
    }

    // process this assembly and move on up the design hierarchy
    o_4b	count = 1;

    visitedComplexIds.insert(id);
    DoNothing();
    if ( !superAssembly.is_null() ) {
	Assembly*	assembly = superAssembly;

	if (!visitedComplexIds.contains(assembly->id)) {
	    count += assembly->traverse7(visitedComplexIds);
        }
    }
    return count;
}
